load("qunit-cli/qunit/qunit/qunit.js");
load("../u1db.js");

var stop_watch = {
    start_time: null, stop_time: null,

    start: function() {
    	this.start_time = new Date();
    },

    stop: function() {
	    this.stop_time = new Date();
    },

    elapsed_seconds: function() {
	    return ( this.stop_time.getMilliseconds() - this.start_time.getMilliseconds() ) / 1000;
    }
};

(function(cli_args) {

    var out = (typeof println !== "undefined") ? println : print;
    var TESTS_PASSED_OUTPUT = [];
    var TESTS_FAILED_OUTPUT = [];

    QUnit.init();
    QUnit.config.blocking = true;
    QUnit.config.autorun = true;
    QUnit.config.updateRate = 0;

    // Hack for Rhino's error objects
    var current_object_parser = QUnit.jsDump.parsers.object;
    QUnit.jsDump.setParser('object', function(obj) {
	    if(typeof obj.rhinoException !== 'undefined') {
	        return obj.name + " { message: '" + obj.message + "', fileName: '" + obj.fileName + "', lineNumber: " + obj.lineNumber + " }";
	    }
	    else {
	        return current_object_parser(obj);
	    }
    });

    var current_test_name = null;
    var current_test_assertions = [];
    var totals = { pass: 0, fail: 0};

    QUnit.testStart = function(name) {
	    current_test_name = name;
	    current_test_assertions = [];
    };

    QUnit.testDone = function(name, fail_count, total_count) {
        var output_this_test = true;
        if (cli_args.length > 0) {
            var found = false;
            for (var i=0; i<cli_args.length; i++) {
                if (cli_args[i] == name) {
                    found = true;
                    break;
                }
            }
            if (!found) output_this_test = false;
        }
	    if(fail_count > 0) {
	        if (output_this_test) {
	            TESTS_FAILED_OUTPUT.push("FAIL ============================ " + name);
	            for(var i = 0; i < current_test_assertions.length; i++) {
        		    TESTS_FAILED_OUTPUT.push("    " + current_test_assertions[i]);
	            }
	        }
	        totals.fail = totals.fail + 1;
	    }
	    else {
	        if (output_this_test) {
    	        TESTS_PASSED_OUTPUT.push("PASS ============================ " + name);
	        }
	        totals.pass = totals.pass + 1;
	    }
    };

    QUnit.log = function(result, message, details) {
	    details.message = details.message || "";

	    var type = (typeof details.expected !== "undefined") ? "EQ" : "OK";

	    var outcome = result ? "PASS" : "FAIL";
	
	    var response = "";
	    if(!result && typeof details.expected !== "undefined") {
	        response = "First parameter: " + JSON.stringify(details.actual) + ", Second parameter: " + JSON.stringify(details.expected);
	    }

	    current_test_assertions.push([outcome, type, details.message, response].join("|"));
    };

    QUnit.done = function() {
	    stop_watch.stop();
	    var i;
        for (i=0; i<TESTS_PASSED_OUTPUT.length; i++) {
            out(TESTS_PASSED_OUTPUT[i]);
        }
        for (i=0; i<TESTS_FAILED_OUTPUT.length; i++) {
            out(TESTS_FAILED_OUTPUT[i]);
        }

	    out("----------------------------------------");
	    out(" PASS: " + totals.pass + "  FAIL: " + totals.fail + "  TOTAL: " + (totals.pass + totals.fail));
	    out(" Finished in " + stop_watch.elapsed_seconds() + " seconds.");
	    out("----------------------------------------");
    };

    QUnit.begin = function() {
	    stop_watch.start();
	    
    }

})(arguments);

// run the tests
load("test_vectorclock.js");
load("test_document.js")
load("test_comparison_helpers.js");
load("test_value_extraction.js");
load("test_error.js")
load("test_sync_internals.js")
load("test_opt_sigs.js")
load("test_lifecycle.js")

_Globals = {};
(function() {
    for (var x in this) {
        _Globals[x] = true;
    }
})();

QUnit.begin(); // hacked b/c currently QUnit.begin is normally called on document.load
QUnit.start();

/* check for escaped vars */
(function() {
    for (var x in this) {
        if (!(x in _Globals)) {
            print("escaped", x);
        }
    }
})();


