from u1db import (
    sync,
    tests,
    )
from u1db.tests import (
    test_backends,
    test_remote_sync_target,
    test_sync,
    )

from bridged import (
    get_u1db_js_bridge,
    BridgedJsObject,
    make_database,
    make_http_sync_target,
    make_synchronizer,
    )


SAME_ORIGIN = True


def startServer(self):
    bridge = get_u1db_js_bridge()
    if not SAME_ORIGIN or bridge.rhino:
        super(self.__class__, self).startServer()
    else:
        self.server = bridge.get_server()
        self.base_url = bridge.get_base_url()
        app = self.make_app()
        bridge.set_fallback_app(app)
        if getattr(app, 'base_url', 1) is None:
            app.base_url = self.base_url


SCENARIOS = [("jsbridged", {
        "make_database_for_test": make_database,
        })]


def skip_test(test):
    test.skipTest("skipped: not applicable/relevant")


class BridgedAllDatabaseTests(test_backends.AllDatabaseTests):

    scenarios = SCENARIOS


class BridgedLocalDatabaseTests(test_backends.LocalDatabaseTests):

    accept_fixed_trans_id = True

    scenarios = SCENARIOS


class BridgedLocalDatabaseValidateSourceGenTests(
    test_backends.LocalDatabaseValidateSourceGenTests):

    scenarios = SCENARIOS


class BridgedLocalDatabaseValidateGenNTransIdTests(
    test_backends.LocalDatabaseValidateGenNTransIdTests):

    scenarios = SCENARIOS

    test_validate_gen_and_trans_id_invalid_gen = skip_test


class BridgedLocalDatabaseWithConflictsTests(
    test_backends.LocalDatabaseWithConflictsTests):

    accept_fixed_trans_id = True

    scenarios = SCENARIOS


# index tests


class BridgedDatabaseIndexTests(test_backends.DatabaseIndexTests):

    scenarios = SCENARIOS


# sync tests


def oauth_http_sync_target(test, path):
    st = make_http_sync_target(test, '~/' + path)
    st.set_oauth_credentials(tests.consumer1.key, tests.consumer1.secret,
                             tests.token1.key, tests.token1.secret)
    return st


class BridgedTestHttpSyncTarget(
    test_remote_sync_target.TestRemoteSyncTargets):

    startServer = startServer

    scenarios = [
        ('http', {'make_app_with_state': test_remote_sync_target.make_http_app,
                  'make_document_for_test': tests.make_document_for_test,
                  'sync_target': make_http_sync_target}),
        ('oauth_http', {'make_app_with_state':
                        test_remote_sync_target.make_oauth_http_app,
                        'make_document_for_test': tests.make_document_for_test,
                        'sync_target': oauth_http_sync_target}),
        ]


def copy_database_for_test(test, db):
    if isinstance(db, BridgedJsObject):
        test.skipTest("js dbs cannot be copied")
    return test_sync.copy_database_for_http_test(test, db)


def sync_for_test(test, db_source, db_target, trace_hook=None,
                  trace_hook_shallow=None):
    if isinstance(db_source, BridgedJsObject):
        if trace_hook:
            test.skipTest("full trace hook not supported by http target")
        path = test._http_at[db_target]
        target = make_http_sync_target(test, path)
        if trace_hook_shallow:
            target._set_trace_hook_shallow(trace_hook_shallow)
        return make_synchronizer(test, db_source, target).sync()
    target = db_target.get_sync_target()
    target._set_trace_hook_shallow(trace_hook or trace_hook_shallow)
    return sync.Synchronizer(db_source, target).sync()


class BridgedDatabaseSyncTests(test_sync.DatabaseSyncTests):

    accept_fixed_trans_id = True

    startServer = startServer

    scenarios = [('jshttp', {
        'make_database_for_test': None, # see create_database_for_role
        'copy_database_for_test': copy_database_for_test,
        'make_document_for_test': tests.make_document_for_test,
        'make_app_with_state': test_sync.make_http_app,
        'do_sync': sync_for_test,
    })]

    def create_database_for_role(self, replica_uid, sync_role):
        if sync_role == 'source':
            js_db = make_database(self, replica_uid)
            js_db._replica_uid = replica_uid
            return js_db
        return test_sync.make_database_for_http_test(self, replica_uid)


class BridgedTestDbSync(test_sync.TestDbSync):

    startServer = startServer

    scenarios = [
        ('jshttp', {
            'make_app_with_state': test_sync.make_http_app,
            'make_database_for_test': make_database,
            }),
        ('jsoauthhttp', {
            'make_app_with_state': test_sync.make_oauth_http_app,
            'make_database_for_test': make_database,
            'oauth': True
            })]


# Use a custom loader to apply the scenarios at load time.
load_tests = tests.load_with_scenarios
