module("deepEqualTest", {
    setup: function() {
        this.db = new U1DB("testing");
    }
});

test("basic", function() {
    ok(U1DB.Utils.deepEqual(null, null));
    ok(U1DB.Utils.deepEqual(1, 1));
    ok(U1DB.Utils.deepEqual(2.0, 1.0*2));
    ok(U1DB.Utils.deepEqual("astring", "astring"));
    ok(U1DB.Utils.deepEqual("a", ""+"a"));
    var x = {}
    ok(U1DB.Utils.deepEqual(x, x));

    ok(!U1DB.Utils.deepEqual(x, null));
    ok(!U1DB.Utils.deepEqual(1, 2.0));
    ok(!U1DB.Utils.deepEqual(2, "2"));
    ok(!U1DB.Utils.deepEqual("a", ""));
})

test("object", function() {
    ok(!U1DB.Utils.deepEqual({a: 1}, 2));
    ok(U1DB.Utils.deepEqual({a: 1, b: true}, {b: true, a: 1}));
    ok(U1DB.Utils.deepEqual({a: 1, b: null}, {b: null, a: 1}));
    ok(!U1DB.Utils.deepEqual({a: 1, b: null}, {a: 1}));
})

test("array", function() {
    ok(U1DB.Utils.deepEqual([1, "2", true], [1, "2", true]));
    ok(!U1DB.Utils.deepEqual([1, 2, 3], null));
    ok(!U1DB.Utils.deepEqual([1, 2, 3], {1: 1, 2: 2, 3: 3}));
    ok(!U1DB.Utils.deepEqual([1, 2, 3], [1, 2]));
    ok(!U1DB.Utils.deepEqual([1, 2, 3], [1, 2, null]));
    ok(U1DB.Utils.deepEqual([{a: 1, b: [{c: 3}]}, false, [5, null]],
                           [{b: [{c: 3}], a: 1}, false, [5, null]]
                          ));
})

module("compareIndexEntries", {
    setup: function() {
        this.db = new U1DB("testing");
    }
});

test("same_length", function() {
    equal(0, this.db._compareIndexEntries([["a"], "x"], [["a"], "y"]));
    equal(-1, this.db._compareIndexEntries([["a"], "x"], [["b"], "y"]));
    equal(1, this.db._compareIndexEntries([["b"], "x"], [["a"], "y"]));

    equal(0, this.db._compareIndexEntries([["a", "k"], "x"],
                                          [["a", "k"], "y"]));
    equal(-1, this.db._compareIndexEntries([["a", "k"], "x"],
                                           [["b", "ka"], "y"]));
    equal(1, this.db._compareIndexEntries([["b", "ka"], "x"], [["a", "k"],
                                                               "y"]));

    /* corner case */
    equal(0, this.db._compareIndexEntries([[], "x"], [[], "y"]));
})

test("prefix", function() {
    equal(1, this.db._compareIndexEntries([["a", "k"], null],
                                          [["a", "ab", "wo"], "y"]));
    equal(-1, this.db._compareIndexEntries([["a", "k"], null],
                                           [["a", "z", "wo"], "y"]));

    equal(-1, this.db._compareIndexEntries([["a"], null],
                                           [["a", "foo"], "y"]));
    equal(-1, this.db._compareIndexEntries([["a", "k"], null],
                                           [["a", "k", "j"], "y"]));

    /* corner case */
    equal(-1, this.db._compareIndexEntries([[], null], [["a", "foo"], "y"]));
})


module("binarySearch", {
    setup: function() {
        this.db = new U1DB("testing");
    }
});

test("greather-than-or-equal", function() {
    var data = [[["a"], "x"], [["b"], "x"], [["c"], "x"], [["d"], "x"],
                [["e"], "x"], [["f"], "x"], [["g"], "x"], [["h"], "x"], 
                [["i"], "x"], [["j"], "x"]];

    for (var i = 0; i<10; i++) {
        var index = this.db._binarySearch(data, [data[i][0], null]);
        equal(i, index);
    }

    for (var i = 0; i<10; i++) {
        var index = this.db._binarySearch(data, [[data[i][0][0]+"1"], null]);
        equal(i, index-1);
    }

    equal(10, this.db._binarySearch(data, [["z"], null]));

    equal(0, this.db._binarySearch(data, [["0"], null]));

    data = [[["a"], "x"], [["b"], "x"], [["b"], "x"], [["b"], "x"],
            [["e"], "x"], [["f"], "x"], [["g"], "x"], [["h"], "x"],
            [["i"], "x"], [["j"], "x"]];

    equal(1, this.db._binarySearch(data, ["b", null]));

})


module("prefix-functions", {
    setup: function() {
        this.db = new U1DB("testing");
    }
});

function e(keys) {
    return [keys, null];
}

test("exactPrefix", function() {
    /* used to generate the upper bound for exact full and prefix searches */
    var db = this.db;
    var cmp = db._compareIndexEntries;
    var exact_pfx = db._exactPrefix;

    /* search for ['a'] */

    equal(-1, cmp(e(['Z']), e(exact_pfx(['a']))));
    equal(-1, cmp(e(['a']), e(exact_pfx(['a']))));
    equal(0, cmp(e(['a\u0000']), e(exact_pfx(['a']))));
    equal(1, cmp(e(['aa']), e(exact_pfx(['a']))));

    equal(-1, cmp(e(['']), e(exact_pfx(['']))));
    equal(0, cmp(e(['\u0000']), e(exact_pfx(['']))));
    equal(1, cmp(e(['a']), e(exact_pfx(['']))));

    /* search for ['0', 'a'] */

    equal(-1, cmp(e(['0', 'Z']), e(exact_pfx(['0', 'a']))));
    equal(-1, cmp(e(['0', 'a']), e(exact_pfx(['0', 'a']))));
    equal(0, cmp(e(['0', 'a\u0000']), e(exact_pfx(['0', 'a']))));
    equal(1, cmp(e(['0', 'aa']), e(exact_pfx(['0', 'a']))));
    equal(1, cmp(e(['01', 'a']), e(exact_pfx(['0', 'a']))));
    equal(1, cmp(e(['1', 'a']), e(exact_pfx(['0', 'a']))));

    /* search for ['0', ''] */

    equal(-1, cmp(e(['0', '']), e(exact_pfx(['0', '']))));
    equal(0, cmp(e(['0', '\u0000']), e(exact_pfx(['0', '']))));
    equal(1, cmp(e(['0', 'a']), e(exact_pfx(['0', '']))));
    equal(1, cmp(e(['01', '']), e(exact_pfx(['0', '']))));
    equal(1, cmp(e(['1', '']), e(exact_pfx(['0', '']))));

    /* search for ['a', '*'] => ['a'] deglobbed */

    equal(-1, cmp(e(['', '']), e(exact_pfx(['a']))));
    equal(-1, cmp(e(['Z', '']), e(exact_pfx(['a']))));
    equal(-1, cmp(e(['Z', 'z']), e(exact_pfx(['a']))));
    equal(-1, cmp(e(['Zz', '']), e(exact_pfx(['a', '']))));
    equal(-1, cmp(e(['a', '']), e(exact_pfx(['a']))));

    equal(1, cmp(e(['ab', '']), e(exact_pfx(['a']))));
    equal(1, cmp(e(['b', '']), e(exact_pfx(['a']))));
    equal(1, cmp(e(['a\u0000', '']), e(exact_pfx(['a']))));

})

test("starPrefix", function() {
    /* used to generate the upper bound for wildcard prefix searches */
    var db = this.db;
    var cmp = db._compareIndexEntries;
    var star_pfx = db._starPrefix;

    /* search for ['a*'] */

    equal(-1, cmp(e(['']), e(star_pfx(['a']))));
    equal(-1, cmp(e(['Z']), e(star_pfx(['a']))));
    equal(-1, cmp(e(['a']), e(star_pfx(['a']))));
    equal(-1, cmp(e(['a\u0000']), e(star_pfx(['a']))));
    equal(-1, cmp(e(['aa']), e(star_pfx(['a']))));

    equal(0, cmp(e(['b']), e(star_pfx(['a']))));
    equal(1, cmp(e(['z']), e(star_pfx(['a']))));

})
