module("U1DBDocument", {
})

test("basic", function() {
    var doc = new U1DBDocument('id', null, {'v': 1});
    equal('id', doc.doc_id);
    equal(null, doc.rev);
    ok(!doc.has_conflicts);
    deepEqual({'v': 1}, doc.content);
    equal('[Document(id, null, {"v":1})]', doc.toString());
    ok(!doc.is_tombstone())
    equal('{"v":1}', doc.get_json())

    doc = new U1DBDocument('id', 'a:1', {});
    equal('a:1', doc.rev);
    equal("[Document(id, a:1, {})]", doc.toString());
    ok(!doc.is_tombstone())
    equal('{}', doc.get_json())

    doc = new U1DBDocument('id', 'a:1', {}, true);
    equal('a:1', doc.rev);
    ok(doc.has_conflicts);
    equal("[Document(id, a:1, conflicted, {})]", doc.toString());
    ok(!doc.is_tombstone())

    doc = new U1DBDocument('id', 'a:1', null);
    equal(null, doc.content);
    equal("[Document(id, a:1, null)]", doc.toString());
    ok(doc.is_tombstone())
    equal(null, doc.get_json())

    doc = new U1DBDocument('veryverylongid', 'a:1|b:2|c:3|d:4|e:5', null);
    equal(null, doc.content);
    equal("[Document(ver...gid, a:1...4|e:5, null)]", doc.toString());

})

test("same_content_as", function() {
    var doc1 = new U1DBDocument('id1', null, {'v': [1,2]});
    var doc2 = new U1DBDocument('id2', null, {'v': [1,2]});
    var doc3 = new U1DBDocument('id3', null, null);
    var doc4 = new U1DBDocument('id3', null, {});

    ok(doc1.same_content_as(doc2));
    ok(doc2.same_content_as(doc1));

    ok(!doc1.same_content_as(doc3));
    ok(!doc3.same_content_as(doc1));
    ok(doc3.same_content_as(doc3));

    ok(!doc4.same_content_as(doc1));
    ok(!doc4.same_content_as(doc3));
    ok(!doc4.same_content_as(doc1));
})

test("set_json", function() {
    var doc = new U1DBDocument('id', null);
    doc.set_json('{"v": [1,2]}');
    deepEqual({"v": [1,2]}, doc.content);

    doc.set_json(null);
    equal(null, doc.content);
})