module("lifecycle", {
    setup: function() {
        this.db = new U1DB("testing");
    }
});

test("lifecycle1", function() {
    var db = this.db;
    db.create_index('1', 'v');
    deepEqual([], db.get_from_index('1', 'a'));
    db.create_doc({'v': 'a'}, 'the-doc');
    var doc = db.get_doc('the-doc');
    deepEqual([doc], db.get_from_index('1', 'a'));
    db.create_index('2', 'v');
    deepEqual([doc], db.get_from_index('2', 'a'));
    doc.content['v'] = 'b';
    db.put_doc(doc);
    deepEqual([], db.get_from_index('2', 'a'));
    deepEqual([doc], db.get_from_index('2', 'b'));
    db.create_index('3', 'v');
    deepEqual([doc], db.get_from_index('3', 'b'));
    db.delete_doc(doc);
    db.create_index('4', 'v');
    deepEqual([], db.get_from_index('1', 'b'));
    deepEqual([], db.get_from_index('2', 'b'));
    deepEqual([], db.get_from_index('3', 'b'));
    deepEqual([], db.get_from_index('4', 'b'));
    doc.content = {'v': 'c'};
    db.put_doc(doc);
    var doc1 = db.get_doc('the-doc');
    deepEqual({'v': 'c'}, doc1.content);
    deepEqual([], db.get_from_index('4', 'a'));
    deepEqual([doc], db.get_from_index('1', 'c'));
    deepEqual([doc], db.get_from_index('4', 'c'));
    db.create_index('5', 'v');
    deepEqual([doc], db.get_from_index('5', 'c'));

})

test("lifecycle2", function() {
    var db = this.db;
    db.create_index('1', 'v');
    var doc = new U1DBDocument('the-doc', null, {'v': 'a'})
    db.put_doc(doc)
    var doc1 = db.get_doc('the-doc');
    deepEqual([doc1], db.get_from_index('1', 'a'));
    db.create_index('2', 'v');
    deepEqual([doc1], db.get_from_index('2', 'a'));
})

test("lifecycle3", function() {
    var db = this.db;
    db.create_index('1', 'v');
    var doc = new U1DBDocument('the-doc', 'other:1', {'v': 'a'})
    db._put_doc_if_newer(doc, true);
    doc = db.get_doc('the-doc');
    deepEqual([doc], db.get_from_index('1', 'a'));
    db.create_index('2', 'v');
    deepEqual([doc], db.get_from_index('2', 'a'));
    doc = new U1DBDocument('the-doc', 'other:2', {'v': 'b'})
    db._put_doc_if_newer(doc, true);
    deepEqual([], db.get_from_index('2', 'a'));
    deepEqual([doc], db.get_from_index('2', 'b'));
    db.create_index('3', 'v');
    deepEqual([doc], db.get_from_index('3', 'b'));
    doc = new U1DBDocument('the-doc', 'other:3', null);
    db._put_doc_if_newer(doc, true);
    db.create_index('4', 'v');
    deepEqual([], db.get_from_index('1', 'b'));
    deepEqual([], db.get_from_index('2', 'b'));
    deepEqual([], db.get_from_index('3', 'b'));
    deepEqual([], db.get_from_index('4', 'b'));
    doc = new U1DBDocument('the-doc', 'other:4', {'v': 'c'});
    db._put_doc_if_newer(doc, true);
    var doc1 = db.get_doc('the-doc');
    deepEqual({'v': 'c'}, doc1.content);
    deepEqual([], db.get_from_index('4', 'a'));
    deepEqual([doc], db.get_from_index('2', 'c'));
    deepEqual([doc], db.get_from_index('4', 'c'));
    db.create_index('5', 'v');
    deepEqual([doc], db.get_from_index('5', 'c'));
})
