module("lifecycle", {
    setup: function() {
        this.db = new U1DB("testing");
    }
});

test("create_doc", function() {
    var doc = this.db.create_doc({'v': 1});
    deepEqual(doc, this.db.get_doc(doc.doc_id));

    doc = this.db.create_doc({'v': 2}, 'my-doc');
    deepEqual(doc, this.db.get_doc('my-doc'));

    doc = this.db.create_doc({'v': 3}, {doc_id: 'my-other-doc'});
    deepEqual(doc, this.db.get_doc('my-other-doc'));
})

test("create_doc_from_json", function() {
    var doc = this.db.create_doc_from_json('{"v": 1}');
    deepEqual(doc, this.db.get_doc(doc.doc_id));

    doc = this.db.create_doc_from_json('{"v": 2}', 'my-doc');
    deepEqual(doc, this.db.get_doc('my-doc'));

    doc = this.db.create_doc('{"v": 3}', {doc_id: 'my-other-doc'});
    deepEqual(doc, this.db.get_doc('my-other-doc'));
})

test("get_doc", function() {
    var doc = this.db.create_doc({});
    this.db.delete_doc(doc);

    deepEqual(null, this.db.get_doc(doc.doc_id));
    deepEqual(doc, this.db.get_doc(doc.doc_id, {include_deleted: true}));
})

test("get_docs", function() {
    var doc1 = this.db.create_doc({'v': 1});
    this.db.delete_doc(doc1);
    var doc2 = this.db.create_doc({'v': 2});
    var doc2_bis = new U1DBDocument(doc2.doc_id, 'other:1', {'v': 5})
    this.db._put_doc_if_newer(doc2_bis, true);
    ok(this.db.get_doc(doc2.doc_id).has_conflicts);

    var docs = this.db.get_docs([doc1.doc_id, doc2.doc_id]);
    equal(1, docs.length);
    equal(doc2.doc_id, docs[0].doc_id);
    ok(docs[0].has_conflicts);

    var docs = this.db.get_docs([doc1.doc_id, doc2.doc_id],
                                {include_deleted: true});
    equal(2, docs.length);
    equal(doc1.doc_id, docs[0].doc_id);
    equal(null, docs[0].content);
    equal(doc2.doc_id, docs[1].doc_id);
    ok(docs[1].has_conflicts);

    var docs = this.db.get_docs([doc1.doc_id, doc2.doc_id],
                                {check_for_conflicts: false});
    equal(1, docs.length);
    equal(doc2.doc_id, docs[0].doc_id);
    ok(!docs[0].has_conflicts);

    var docs = this.db.get_docs([doc1.doc_id, doc2.doc_id],
                                {check_for_conflicts: false,
                                 include_deleted: true});
    equal(2, docs.length);
    equal(doc1.doc_id, docs[0].doc_id);
    equal(null, docs[0].content);
    equal(doc2.doc_id, docs[1].doc_id);
    ok(!docs[1].has_conflicts);
})

test("whats_changed", function() {
    var doc1 = this.db.create_doc({'v': 1});
    var doc2 = this.db.create_doc({'v': 2});

    deepEqual([2, '', [[doc1.doc_id, 1, ''], [doc2.doc_id, 2, '']]],
              this.db.whats_changed());

    deepEqual([2, '', [[doc2.doc_id, 2, '']]], this.db.whats_changed(1));

    deepEqual([2, '', [[doc2.doc_id, 2, '']]],
              this.db.whats_changed({old_generation: 1}));

})

test("get_all_docs", function() {
    var doc1 = this.db.create_doc({'v': 1});
    this.db.delete_doc(doc1);
    var doc2 = this.db.create_doc({'v': 2});
    var doc2_bis = new U1DBDocument(doc2.doc_id, 'other:1', {'v': 5})
    this.db._put_doc_if_newer(doc2_bis, true);
    ok(this.db.get_doc(doc2.doc_id).has_conflicts);

    var gen_docs = this.db.get_all_docs();
    equal(1, gen_docs[1].length);

    var gen_docs = this.db.get_all_docs({include_deleted: true});
    equal(2, gen_docs[1].length);

    var gen_docs = this.db.get_all_docs({include_deleted: false});
    equal(1, gen_docs[1].length);
})
