module("_parse_sync_stream", {
    setup: function() {
        this.tgt = new U1DBHTTPSyncTarget(null);
    }
});

assertRaises = function(fn, error, message) {
    try {
        fn();
        ok(false, "expected "+ error.prototype.name);
    } catch(e) {
        if(!(e instanceof error)) {
            ok(false, "expected "+ error.prototype.name + " got " + e)
        }
    }
}


test("wrong_start", function() {
    var tgt = this.tgt;
    assertRaises(function() {
        tgt._parse_sync_stream("{}\r\n]", null);
    }, U1DB.Errors.BrokenSyncStream);

    assertRaises(function() {
        tgt._parse_sync_stream("\r\n{}\r\n]", null);
    }, U1DB.Errors.BrokenSyncStream);

    assertRaises(function() {
        tgt._parse_sync_stream("", null);
    }, U1DB.Errors.BrokenSyncStream);
})


test("wrong_end", function() {
    var tgt = this.tgt;

    assertRaises(function() {
        tgt._parse_sync_stream("[\r\n{}", null);
    }, U1DB.Errors.BrokenSyncStream);

    assertRaises(function() {
        tgt._parse_sync_stream("[\r\n", null);
    }, U1DB.Errors.BrokenSyncStream);

})

test("missing_comma", function() {
    var tgt = this.tgt;
    assertRaises(function() {
        tgt._parse_sync_stream('[\r\n{}\r\n{"id": "i", "rev": "r", ' +
                               '"content": "c", "gen": 3}\r\n]', null);
    }, U1DB.Errors.BrokenSyncStream);
})

test("no_entries", function() {
    var tgt = this.tgt;

    assertRaises(function() {
        tgt._parse_sync_stream("[\r\n]", null);
    }, U1DB.Errors.BrokenSyncStream);
})

test("extra_comma", function() {
    var tgt = this.tgt;

    assertRaises(function() {
        tgt._parse_sync_stream("[\r\n{},\r\n]", null);
    }, U1DB.Errors.BrokenSyncStream);

    assertRaises(function() {
        tgt._parse_sync_stream(
            '[\r\n{},\r\n{"id": "i", "rev": "r", ' +
                '"content": "{}", "gen": 3, "trans_id": "T-sid"}' +
                ',\r\n]', function() {});
    }, U1DB.Errors.BrokenSyncStream);

})

test("error_in_stream", function() {
    var tgt = this.tgt;

    assertRaises(function() {
        tgt._parse_sync_stream('[\r\n{"new_generation": 0},' +
                               '\r\n{"error": "unavailable"}\r\n', null);
    }, U1DB.Errors.Unavailable);

    assertRaises(function() {
        tgt._parse_sync_stream('[\r\n{"error": "unavailable"}\r\n', null);
    }, U1DB.Errors.Unavailable);

    assertRaises(function() {
        tgt._parse_sync_stream('[\r\n{"error": "?"}\r\n', null);
    }, U1DB.Errors.BrokenSyncStream);

})
