module("value-extraction", {
    setup: function() {
        this.db = new U1DB("testing");
    }
});

test("doGet", function() {
    var doc = {content: {'foo': 'x', 'bar': {'baz': 'y'}}}

    var d = this.db._parseIndexExpressions(['foo']);
    deepEqual([['x']], this.db._getIndexEntriesForDoc({trees: d}, doc));

    d = this.db._parseIndexExpressions(['foo', 'bar.baz']);
    deepEqual([['x', 'y']], this.db._getIndexEntriesForDoc({trees: d}, doc));

})

test("number", function() {
    var d = this.db._parseIndexExpressions(['number(foo, 2)']);

    var doc = {content: {'foo': 3} }
    deepEqual([['03']], this.db._getIndexEntriesForDoc({trees: d}, doc));

    doc = {content: {'foo': null} }
    deepEqual([], this.db._getIndexEntriesForDoc({trees: d}, doc));

    doc = {content: {'foo': '3' } }
    deepEqual([], this.db._getIndexEntriesForDoc({trees: d}, doc));
})

test("bool", function() {
    var d = this.db._parseIndexExpressions(['bool(foo)']);

    var doc = {content: {'foo': true} }
    deepEqual([['1']], this.db._getIndexEntriesForDoc({trees: d}, doc));

    var doc = {content: {'foo': false} }
    deepEqual([['0']], this.db._getIndexEntriesForDoc({trees: d}, doc));

    doc = {content: {'foo': null} }
    deepEqual([], this.db._getIndexEntriesForDoc({trees: d}, doc));

    doc = {content: {'foo': '1' } }
    deepEqual([], this.db._getIndexEntriesForDoc({trees: d}, doc));
})

test("lower/upper", function() {
    var d = this.db._parseIndexExpressions(['lower(foo)', 'upper(foo)']);

    var doc = {content: {'foo': 'aB'} }
    deepEqual([['ab', 'AB']], this.db._getIndexEntriesForDoc({trees: d}, doc));

    doc = {content: {'foo': 1 } }
    deepEqual([], this.db._getIndexEntriesForDoc({trees: d}, doc));

    var doc = {content: {'foo': false} }
    deepEqual([], this.db._getIndexEntriesForDoc({trees: d}, doc));

    doc = {content: {'foo': null} }
    deepEqual([], this.db._getIndexEntriesForDoc({trees: d}, doc));

})

test("split_words", function() {
    var d = this.db._parseIndexExpressions(['split_words(foo)']);

    var doc = {content: {'foo': ' a  BC d'} }
    deepEqual([['a'], ['BC'], ['d']],
              this.db._getIndexEntriesForDoc({trees: d}, doc));

    doc = {content: {'foo': 1 } }
    deepEqual([], this.db._getIndexEntriesForDoc({trees: d}, doc));

    var doc = {content: {'foo': false} }
    deepEqual([], this.db._getIndexEntriesForDoc({trees: d}, doc));

    doc = {content: {'foo': null} }
    deepEqual([], this.db._getIndexEntriesForDoc({trees: d}, doc));

})
