module("TestVectorClockRev", {
    setup: function() {
        this.assertIsNewer = function(newer_rev, older_rev) {
            var new_vcr = new U1DBVectorClock(newer_rev);
            var old_vcr = new U1DBVectorClock(older_rev);
            ok(new_vcr.is_newer(old_vcr), newer_rev + " newer than " + older_rev);
            ok(!old_vcr.is_newer(new_vcr), older_rev + " not newer than " + newer_rev);
        };
        this.assertIsConflicted = function(rev_a, rev_b) {
            var vcr_a = new U1DBVectorClock(rev_a);
            var vcr_b = new U1DBVectorClock(rev_b);
            ok(!vcr_a.is_newer(vcr_b), rev_a + " not newer than " + rev_b);
            ok(!vcr_b.is_newer(vcr_a), rev_b + " not newer than " + rev_a);
        };
        this.assertRoundTrips = function(rev) {
            equal(rev, (new U1DBVectorClock(rev)).as_str());
        };
        this.assertIncrement = function(original, replica_uid, after_increment) {
            var vcr = new U1DBVectorClock(original);
            vcr.increment(replica_uid);
            equal(after_increment, vcr.as_str());
        };
        this.assertMaximize = function(rev1, rev2, maximized) {
            var vcr1 = new U1DBVectorClock(rev1);
            var vcr2 = new U1DBVectorClock(rev2);
            vcr1.maximize(vcr2);
            equal(maximized, vcr1.as_str());
            // reset vcr1 to maximize the other way
            vcr1 = new U1DBVectorClock(rev1);
            vcr2.maximize(vcr1);
            equal(maximized, vcr2.as_str());
        };
    }
});

test("nil", function() {
    equal("", (new U1DBVectorClock(null)).as_str());
});

test("_is_newer_doc_rev", function() {
    this.assertIsNewer('test:1', null)
    this.assertIsNewer('test:2', 'test:1')
    this.assertIsNewer('other:2|test:1', 'other:1|test:1')
    this.assertIsNewer('other:1|test:1', 'other:1')
    this.assertIsNewer('a:2|b:1', 'b:1')
    this.assertIsNewer('a:1|b:2', 'a:1')
    this.assertIsConflicted('other:2|test:1', 'other:1|test:2')
    this.assertIsConflicted('other:1|test:1', 'other:2')
    this.assertIsConflicted('test:1', 'test:1')
});

test("round_trips", function() {
    this.assertRoundTrips("test:1");
    this.assertRoundTrips("a:1|b:2");
    this.assertRoundTrips("alternate:2|test:1");
});

test("handles_sort_order", function() {
    equal("a:1|b:2", (new U1DBVectorClock("b:2|a:1")).as_str());
    equal("a:1|b:2|c:3|d:4|e:5|f:6", (new U1DBVectorClock("f:6|a:1|b:2|c:3|d:4|e:5")).as_str());
    equal("a:1|b:2|c:3|d:4|e:5|f:6", (new U1DBVectorClock("f:6|e:5|d:4|c:3|b:2|a:1")).as_str());
});

test("increment", function() {
    this.assertIncrement(null, 'test', 'test:1');
    this.assertIncrement('test:1', 'test', 'test:2');
});
test("increment_adds_uid", function() {
    this.assertIncrement('other:1', 'test', 'other:1|test:1');
    this.assertIncrement('a:1|ab:2', 'aa', 'a:1|aa:1|ab:2');
});
test("increment_update_partial", function() {
    this.assertIncrement('a:1|ab:2', 'a', 'a:2|ab:2');
    this.assertIncrement('a:2|ab:2', 'ab', 'a:2|ab:3');
});
test("increment_appends_uid", function() {
    this.assertIncrement('b:2', 'c', 'b:2|c:1');
});
test("maximize", function() {
    this.assertMaximize(null, null, '')
    this.assertMaximize(null, 'x:1', 'x:1')
    this.assertMaximize('x:1', 'y:1', 'x:1|y:1')
    this.assertMaximize('x:2', 'x:1', 'x:2')
    this.assertMaximize('x:2', 'x:1|y:2', 'x:2|y:2')
    this.assertMaximize('a:1|c:2|e:3', 'b:3|d:4|f:5',
                        'a:1|b:3|c:2|d:4|e:3|f:5')
});

